#ifndef HELP_H
#define HELP_H

const char USAGE[] = {
	"\nConvert impulse tracker modules to an SNES SPC700 sound file or soundbank.\n"
	"Usage: smconv [options] [input]\n\n"
};

const char OPTIONS[] = {
	"Mandatory arguments to long options are mandatory for short options too.\n"
	"  -d, --driver [driver]     [driver] may be one of the following:\n"
	"                            celes     - uses many IT commands for other\n"
	"                                        features. it has no sfx support.\n"
	"                            snesmod*  - the default sound driver.\n"
	"                            pitchmod  - pitch modulation and noise generation.\n"
	"                            supernofx - special features and no sfx support.\n"
	"                            abnofx\n"
	"                            abfm\n" 
        "                            abstream  - ab version of drivers\n"
	"                            help      - detailed description of drivers.\n\n"
	"  -h, --hirom               Use HIROM mapping mode for soundbank.\n\n"
	"  -i, --info                Provide driver information.\n\n"
	"  -o, --output [file]       Specify output file or file base. This is required\n"
	"                            for soundbank mode.\n\n"
	"  -s, --soundbank           Create a soundbank instead of creating an SPC.\n\n"
	"                            This mode allows for multiple songs.\n"
	"  -v, --verbose             Enable verbose output to view how much RAM the\n\n"
	"                            modules will use.\n\n"
	"      --help                Show Help.\n"
	"      --doc                 Show SNESMod, PitchMod and SuperNOFX composition\n"
	"                            documentation. Include -d, --driver and a specific\n"
	"                            driver to its documentation.\n\n"
	"EXAMPLES:\n"
	"  smconv --doc -d celes                                 # Show Celes documentation\n"
	"  smconv -s -o build/soundbank -h song1.it song2.it     # Two song soundbank\n"
	"  smconv -s -o build/soundbank -h -d supernofx input.it # supernofx driver\n"
	"  smconv input.it                                       # IT->SPC\n"
};

const char DRIVER_INFO[] = {
	"DRIVER:        DESCRIPTION:\n"
	"  snesmod      SNESMod is the default driver when the [-d|--driver] option\n"
	"               isn't used. This is Mukunda's original driver with a couple of\n"
	"               bug fixes and support for vibrato waveform (S3x)\n\n"
	"  celes        This driver uses many IT commands for other features.\n\n"
	"  pitchmod     This driver has the same features as snesmod as well as support\n"
        "               for pitch modulation and noise generation which were added by\n"
	"               KungFuFurby. Special care must be taken if you use these\n"
	"               features and have sound effects.\n\n"
	"  supernofx    In addition to pitchmod features, Super SNESMod lets you\n"
	"               modify echo delay, echo feedback, and the echo coeffiecents. It\n"
	"               does not have support for sound effects.\n\n"
	"  abnofx       Default Augutus version. Some on/off commands are swapped.\n"
	"               This driver does not have sound effect support.\n\n"
	"  abfm         ABNoFX with psychopathicteen FM generation code.\n\n"
	"  abmes        An old version of abnofx. Set master volume (uxx) > 80h to\n"
	"               gradually decrease the requested master volume level (-80h)\n"
	"		while gradually increasing the echo volume to the default\n"
	"               master volume level. Speed is based on evol time setting\n"
	"               (Kxx) > 80h subtract 80h for the evol delay time.\n\n"
	"  abspc        SPC version of abnofx.\n\n"
	"  abstream     abnofx with support for sound effects. Other features have\n"
	"               been removed to save RAM.\n\n"
	"  smconv --doc -d [driver]                              # Display driver commands\n\n"
};
	
const char RULES[] = {
	".-----------------------------------------------------------------------.\n"
	"|                       SNESMOD COMPOSITION RULES                       |\n"
	"|-----------------------------------------------------------------------|\n"
	"|                                                                       |\n"
	"| Impulse tracker format must be used. 8 Channels MAX.                  |\n"
	"|                                                                       |\n"
	"| Notes can not exceed 128KHz playback rate.                            |\n"
	"|                                                                       |\n"
	"| The module must be in instrument mode and must have instruments.      |\n"
	"|                                                                       |\n"
	"| DO NOT USE UNDEFINED INSTRUMENTS, they will not silence the channel,  |\n"
	"| THEY WILL CAUSE MALFUNCTION!                                          |\n"
	"|                                                                       |\n"
	"| The 64 or less samples must fit within 58K of memory. This is AFTER   |\n"
	"| BRR compression. 8-bit samples will be reduced to 9/16 size. 16-bit   |\n"
	"| samples will be reduced to 9/32 size.                                /\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                USAGE                                |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| To use alternate versions of SNESMOD edit DRIVER in your Makefile.  |\n"
	"|                                                                     |\n"
	"| DRIVER		:= PITCHMOD                                   |\n"
	"| ASFLAGS		:= --cpu 65815 -S -D HIROM -D $(DRIVER)       |\n"
	"| SMCONVFL		:= --hirom --driver $(DRIVER) --soundbank \\   |\n"
	"|                          --output soundbank                         |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                            CUT FEATURES                             |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
};

const char SNESMOD_DOC[] = {
	"| This is not a complete IT implementation, here are some IT features |\n"
	"| that you must not use:                                              |\n"
	"|                                                                     |\n"
	"| * New Note Actions are not supported.                               |\n"
	"| * Complex sample mapping is not supported.                          |\n"
	"|     Only the center entry in each instruments sample map will be    |\n"
	"|     used for the entire map! (and the note values are ignored!!!)   |\n"
	"| * Pitch envelope is not supported.                                  |\n"
	"| * No stereo samples!!                                               |\n"
	"| * Envelope sustain must remain on one node only.                    |\n"
	"| * Envelope nodes must be only 1 to 256 ticks apart.                 |\n"
	"| * Envelope carry is not supported.                                  |\n"
	"| * Random volume/panning variations are not supported.               |\n"
	"| * Pitch/Pan separation is not supported.                            |\n"
	"| * 'Old effects' must be set to 'off'.                               |\n"
	"| * Only linear frequency mode is supported.                          |\n"
	"| * Gxx must not share memory with Exx/Fxx.                           |\n"
	"| * Auto-vibrato is not supported.                                    |\n"
	"| * Filters are not supported.                                        |\n"
	"| * These volume effects are not supported:                           |\n"
	"|     Ex, Fx, Gx, Hx                                                  |\n"
	"| * These effects are partially/not supported:                        |\n"
	"|     Axx - Set song speed (hex)                                      |\n"
	"|     Bxx - Jump to Order (hex)                                       |\n"
	"|     Cxx - Pattern Break                                             |\n"
	"|     Dxy - Volume slide                                              |\n"
	"|     Exy - Pitch slide down                                          |\n"
	"|     Fxy - Pitch slide up                                            |\n"
	"|     Gxx - Glissando                                                 |\n"
	"|     Hxy - Vibrato                                                   |\n"
	"|     Ixx - Tremor                 NOT SUPPORTED                      |\n"
	"|     Jxy - Arpeggio                                                  |\n"
	"|     Kxy - Volume slide+Vibrato                                      |\n"
	"|     Lxy - Volume slide+Glissando NOT SUPPORTED                      |\n"
	"|     Mxx - Set channel volume                                        |\n"
	"|     Nxx - Channel volume slide                                      |\n"
	"|     Oxx - Sample offset          NOT SUPPORTED                      |\n"
	"|     Pxy - Panning slide                                             |\n"
	"|     Qxy - Retrigger note                                            |\n"
	"|     Rxy - Tremolo                NOT SUPPORTED                      |\n"
	"|     S0x - Set filter             NOT SUPPORTED, REPURPOSED          |\n"
	"|     S1x - Set glissando ctrl     NOT SUPPORTED, REPURPOSED          |\n"
	"|     S2x - Set finetine           NOT SUPPORTED, REPURPOSED          |\n"
	"|     S33 - Random vibrato wave    NOT SUPPORTED, REPURPOSED          |\n"
	"|     S43 - Random tremolo wave    NOT SUPPORTED, REPURPOSED          |\n"
	"|     S5x - Set panbrello wave     NOT SUPPORTED                      |\n"
	"|     S6x - Pattern delay (ticks)  NOT SUPPORTED                      |\n"
	"|     S7x - New note action        NOT SUPPORTED                      |\n"
	"|     S8x - Set panning position                                      |\n"
	"|     S9x - not supported          NOT SUPPORTED, REPURPOSED          |\n"
	"|     SAy - not supported          NOT SUPPORTED, REPURPOSED          |\n"
	"|     SBx - Pattern loop           NOT SUPPORTED                      |\n"
	"|     SCx - Note cut                                                  |\n"
	"|     SDx - Note delay                                                |\n"
	"|     SEx - Pattern delay (row)    NOT SUPPORTED                      |\n"
	"|     SFx - Set parameter MIDI     NOT SUPPORTED, REPURPOSED          |\n"
	"|     T0y - Tempo slide down                                          |\n"
	"|     T1y - Tempo slide up                                            |\n"
	"|     Txy - Tempo                                                     |\n"
	"|     Uxy - Fine vibrato           NOT SUPPORTED                      |\n"
	"|     Vxy - Global volume                                             |\n"
	"|     Wxy - Global volume slide                                       |\n"
	"|     Yxy - Panbrello              NOT SUPPORTED                      |\n"
	"|     Zxx - MIDI Macros            NOT SUPPORTED, REPURPOSED          |\n"
	"|                                                                     |\n"
	"| And because mukunda is so damn lazy these are also not supported:   |\n"
	"|  panning envelopes                                                  |\n"
	"|                                                                     |\n"
	"| * '+++' patterns ARE supported!!!                                   |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::. ........... .:::::::::::::::::::::::::::|\n"
	"|  ENABLE ADSR AND SET AD  ::             ::  ENABLE ADSR AND SET AD  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Mxx ::  SNESMod    Pitchmod                                         |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"|    M7F - Use direct gain mode for the current channel [default]     |\n"
	"| M80-FF - Enable ADSR for current channel and set the attack and     |\n"
	"|          decay rate (%%Adddaaaa)                                     |\n"
	"|                                                                     |\n"
	"| * Volume commands will be ignored                                   |\n"
	"|                                                                     |\n"
        "| * M00-40 sets channel volume per IT spec                            |\n"
	"|                                                                     |\n"
	"|:::::::::. ...............................................  .::::::::|\n"
	"|  ECHO  ::                                                 ::  ECHO  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::  SNESMod    Pitchmod                     Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S01 - Turn on echo for channel                                      |\n"
	"| S02 - Turn off echo for channel                                     |\n"
	"| S03 - Turn on echo for all channels                                 |\n"
	"| S04 - Turn off echo for all channels                                |\n"
	"|                                                                     |\n"
	"| * The other echo settings for the module can be configured during   |\n"
	"|   conversion. See 'SPECIAL SONG MESSAGE COMMANDS'                   |\n"
	"|                                                                     |\n"
	"| * When setting the Echo Delay parameter, you lose '2K x D' of       |\n"
	"|   sample memory from the SPC.                                       |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::. ....................... .:::::::::::::::::::::|\n"
	"|  PITCH MODULATION  ::                         ::  NOISE GENERATION  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::             Pitchmod                     Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S05 - Turn on pitch modulation for one channel                      |\n"
	"| S06 - Turn off pitch modulation for one channel                     |\n"
	"| S07 - Turn on pitch modulation for all channels                     |\n"
	"| S08 - Turn off pitch modulation for all channels                    |\n"
	"| S09 - Turn on noise generation for one channel                      |\n"
	"| S0A - Turn off noise generation for one channel                     |\n"
	"| S0B - Turn on noise generation for all channels                     |\n"
	"| S0C - Turn off noise generation for all channels                    |\n"
	"|                                                                     |\n"
	"| * The global volume should always be maximum, or else your pitch    |\n"
	"|   modulation might be weak.                                         |\n"
	"|                                                                     |\n"
	"|::::::::::. ............................................. .::::::::::|\n"
	"|  RESET  ::                                               ::  RESET  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S0D - Reset echo delay to default defined in the song message       |\n"
	"| S0E - Reset echo volume to default defined in the song message      |\n"
	"| S0F - Reset echo feedback to default defined in the song message    |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY  ::                           ::  NOISE FREQUENCY  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S1x-S2x ::         Pitchmod                     Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S1x-S2x - Noise frequency, -10h to get your frequency. This         |\n"
	"|           automatically disables noise frequency sweep if using     |\n"
	"|           Super SNESMod NoFX.                                       |\n"
	"|                                                                     |\n"
	"| * Applies to all channels that have noise enabled.                  |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::. ....................... .:::::::::::::::::::::|\n"
	"|  VIBRATO WAVEFORM  ::                         ::  VIBRATO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x  :: SNESMod     Pitchmod                     Super SNESMod NoFX |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S30 - Sine [default]                                                |\n"
	"| S31 - Ramp down                                                     |\n"
	"| S32 - Square                                                        |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x  ::             Pitchmod                     Super SNESMod NoFX |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S34 - Triangle                                                      |\n"
	"| S35 - Ramp Up                                                       |\n"
	"| S36 - Square 2                                                      |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  ECHO WRITE FLAG  ::                           ::  ECHO WRITE FLAG  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x  ::                                          Super SNESMod NoFX |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S37 - Disable echo writes                                           |\n"
	"| S38 - Enable echo writes                                            |\n"
	"|                                                                     |\n"
	"|::::::::::::::. .................................... .:::::::::::::::|\n"
	"|  SET FILTER ::                                      ::  SET FILTER  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
        "| S39 - Reset all coefficients, master and echo volume, echo feedback |\n"
        "|       and echo delay                                                |\n"
	"| S3A - Set default filter                                            |\n"
	"| S3B - Set bandpass filter                                           |\n"
	"| S3C - Set highpass filter                                           |\n"
	"| S3D - Set lowpass filter                                            |\n"
	"| S3E - Set ? filter (ren & stimpy)                                   |\n"
	"| S3F - Set ? filter (star ocean)                                     |\n"
	"|                                                                     |\n"
	"| * If S96 or S97 has been set then S3[A-F] will sweep to these       |\n"
	"|   preset filter values instead of immediately setting them          |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::. ............... .:::::::::::::::::::::::::|\n"
	"|  SET TREMOLO WAVEFORM  ::                 ::  SET TREMOLO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S4x ::          ::          ::               :: Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S40 - Sine [default]                                                |\n"
	"| S41 - Ramp down                                                     |\n"
	"| S42 - Square                                                        |\n"
	"|                                                                     |\n"
	"| * See note after panbrello commands.                                |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  MISC COMMANDS  ::                               ::  MISC COMMANDS  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S4x ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S44 - EVOL max -> min                                               |\n"
	"| S45 - EVOL min -> max [default]                                     |\n"
	"| S46 - Enable swing tempo (Use S9D, ZF1-FF for different speeds. The |\n"
	"|       default is 1.)                                                |\n"
	"| S47 - Disable swing tempo [default]                                 |\n"
	"| S48 - Enable wavetable cycle. This replaces the first sample, which |\n"
	"|       should have a sample rate of 16744Hz with a length of 32,     |\n"
	"|       with wavetable samples. Up to 8 brr samples can be uploaded   |\n"
	"|       using spcChangeWaveTable.                                     |\n"
	"| S49 - Disable wavetable cycle [default] It is not possible to       |\n"
	"|       restore the original sample at this time.                     |\n"
	"| S4A - Noise frequency increase                                      |\n"
	"| S4B - Noise frequency decrease [default]                            |\n"
	"| S4C - Noise frequency repeat on                                     |\n"
	"| S4D - Noise frequency repeat off [default]                          |\n"
	"| S4E - Noise frequency ping pong on                                  |\n"
	"| S4F - Noise frequency ping pong off [default]                       |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::. ............ .::::::::::::::::::::::::::|\n"
	"|  SET PANBRELLO WAVEFORM  ::              :: SET PANBRELLO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S5x ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S50 - Sine [default]                                                |\n"
	"| S51 - Ramp down                                                     |\n"
	"| S52 - Square                                                        |\n"
	"|                                                                     |\n"
	"| * Vibrato, tremolo, and panbrello waveform may be set using S3x.    |\n"
	"|   The S4x and S5x commands are available if you need to hear what   |\n"
	"|   it will sound like in the tracker. When using Hxx, Rxx, and Yxx   |\n"
	"|   on the same channel they will use the same parameters if          |\n"
	"|   unspecified. If you use H42 for vibrato and later use R75 for     |\n"
	"|   tremolo, using H00 after the tremolo command is the same as using |\n"
        "|   H75.                                                              |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::. ............. .::::::::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY SWEEP  ::               ::  NOISE FREQUENCY SWEEP  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S5x ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S54-S5F - Start noise frequency sweep [-40h = starting frequency]   |\n"
	"|                                                                     |\n"
	"| * This is a shortcut for S9D ZMODE Z14-Z1F. Use ZDA-ZDF to alter    |\n"
	"|   the behavior of this command.                                     |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SOUND CONTROL  ::                               ::  SOUND CONTROL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::  SNESMod    Pitchmod                     Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S91 - Surround sound                                                |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::. ................................ .::::::::::::::::|\n"
	"|  MUTE CHANNEL  ::                                  :: MUTE CHANNEL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::          :: Pitchmod ::               :: Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S92 - Mute channel for pitch modulation                             |\n"
	"| S93 - Unmute channel                                                |\n"
	"|                                                                     |\n"
	"| * It is not necessary to mute the channel providing the pitch       |\n"
	"|   factor.                                                           |\n"
	"|                                                                     |\n"
	"|::::::::::::::. ..................................... .::::::::::::::|\n"
	"|  SET ZMODE  ::                                       ::  SET ZMODE  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S96 - Set ZMODE to channel filter sweep and reset EDL, EFB, EFIR,   |\n"
	"|       EVOL.                                                         |\n"
	"| S97 - Set ZMODE to channel filter sweep. Use Zxx to sweep to value  |\n"
	"|       xx on the current channel, or use S3[A-F] to sweep all        |\n"
	"|       all channels to pre-set filter values.                        |\n"
	"| S98 - Set ZMODE to modify the coefficients for the 8-tap FIR filter |\n"
	"| S99 - Set ZMODE to modify the master volume                         |\n"
	"| S9A - Set ZMODE to modify echo volume                               |\n"
	"| S9B - Set ZMODE to modify echo feedback                             |\n"
	"| S9C - Set ZMODE to modify echo delay and for special commands       |\n"
	"| S9D - Set ZMODE to modify echo volume sweep and noise sweep         |\n"
	"| S9E - Set ZMODE to modify echo sweep minimum and maximum values     |\n"
	"| S9F - Set ZMODE to enable filter sweep and change filter delay      |\n"
	"|                                                                     |\n"
        "|:::::::::::::::::::::::::::::. ....... .:::::::::::::::::::::::::::::|\n"
        "|  SET SUSTAIN RATE & LEVEL  ::         ::  SET SUSTAIN RATE & LEVEL  |\n"
        "|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
        "| Zxx ::  SNESMod :: Pitchmod ::               ::                     |\n"
        "|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| Z00-FF - Set sustain rate and level for the current channel         |\n"
	"|          (%%rrrlllll)                                                |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SET PARAMETER  ::                               ::  SET PARAMETER  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Zxx ::                                          Super SNESMod NoFX  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
        "| [S96|S97]                                                           |\n"
	"|       Z80    - disable channel filter sweep for current channel     |\n"
	"|       Zxx    - sweep to this value* on the current channel          |\n"
	"|                                                                     |\n"
	"| [S98] Z80    - Reset coefficients to default                        |\n"
	"|       Zxx    - Set coefficient value* on the current channel        |\n"
	"|                                                                     |\n"
	"| [S99] Z80    - Reset master volume to default set in source code    |\n"
	"|       Zxx    - Set master volume                                    |\n"
	"|                Values >80h will slide master volume to echo volume  |\n"
	"|                level and echo volume to master volume level.        |\n"
	"|                                                                     |\n"
	"| [S9A] Z80    - Reset echo volume                                    |\n"
	"|       Zxx    - Set echo volume*                                     |\n"
	"|                                                                     |\n"
	"| [S9B] Z80    - Reset echo feedback                                  |\n"
	"|       Zxx    - Set echo feedback*                                   |\n"
	"|                                                                     |\n"
	"| [S9C] Z00-0F - Set echo delay                                       |\n"
	"|       Z10    - Reset all coefficients, master and echo volume, echo |\n"
	"|                feedback and echo delay                              |\n"
	"|       Z11-4F - Set delay between wavetable cycle update [-10h,x*4]  |\n"
	"|       Z50-57 - Set minimum wavetable [-50h]                         |\n"
	"|       Z58-5F - Set maximum wavetable [-58h]                         |\n"
	"|                                                                     |\n"
	"| [S9D] Z00-1F - Start noise frequency sweep                          |\n"
	"|       Z20-3F - Set minimum noise frequency value [-20h]             |\n"
	"|       Z40-5F - Set maximum noise frequency value [-40h]             |\n"
	"|       Z60    - Disable noise frequency sweep [default]              |\n"
	"|       Z61-D9 - Set noise frequency sweep speed                      |\n"
	"|       ZDA    - Noise frequency increase                             |\n"
	"|       ZDB    - Noise frequency decrease [default]                   |\n"
        "|       ZDC    - Noise frequency repeat on                            |\n"
	"|       ZDD    - Noise frequency repeat off [default]                 |\n"
	"|       ZDE    - Noise frequency ping pong on                         |\n"
	"|       ZDF    - Noise frequency ping pong off [default]              |\n"
	"|       ZE0    - Disable echo volume sweep                            |\n"
	"|       ZE1-EF - Enable echo volume sweep with speed x                |\n"
	"|       ZF0    - Disable swing tempo                                  |\n"
	"|       ZF1-FF - Enable swing tempo; mod_speed+x, mod_speed [-F0h]    |\n"
	"|                                                                     |\n"
	"| [S9E] Z00-7F - set minimum echo volume for filter sweep             |\n"
	"|       Z80-FF - set maximum echo volume for filter sweep             |\n"
	"|                                                                     |\n"
	"| [S9F] Z00    - Disable filter sweep                                 |\n"
	"|       Z01-FF - Delay before changing filter coefficients            |\n"
	"|                [default 80h]                                        |\n"
	"|                                                                     |\n"
	"| * Values >80h are negative numbers.                                 |\n"
	".                                                                     .\n"
};

const char AB_DOC[] = {
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                            CUT FEATURES                             |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| This is not a complete IT implementation, here are some IT features |\n"
	"| that you must not use:                                              |\n"
	"|                                                                     |\n"
	"| * New Note Actions are not supported.                               |\n"
	"| * Complex sample mapping is not supported.                          |\n"
	"|     Only the center entry in each instruments sample map will be    |\n"
	"|     used for the entire map! (and the note values are ignored!!!)   |\n"
	"| * Pitch envelope is not supported.                                  |\n"
	"| * No stereo samples!!                                               |\n"
	"| * Envelope sustain must remain on one node only.                    |\n"
	"| * Envelope nodes must be only 1 to 256 ticks apart.                 |\n"
	"| * Envelope carry is not supported.                                  |\n"
	"| * Random volume/panning variations are not supported.               |\n"
	"| * Pitch/Pan separation is not supported.                            |\n"
	"| * 'Old effects' must be set to 'off'.                               |\n"
	"| * Only linear frequency mode is supported.                          |\n"
	"| * Gxx must not share memory with Exx/Fxx.                           |\n"
	"| * Auto-vibrato is not supported.                                    |\n"
	"| * Filters are not supported.                                        |\n"
	"| * Panning envlopes are not supported.                               |\n"
	"| * These volume effects are not supported:                           |\n"
	"|     Ex, Fx, Gx, Hx                                                  |\n"
	"|                                                                     |\n"
	"|     Axx - Set song speed (hex)                                      |\n"
	"|     Bxx - Jump to Order (hex)                                       |\n"
	"|     Cxx - Pattern Break          NOT SUPPORTED, REPURPOSED          |\n"
	"|     Dxy - Volume slide                                              |\n"
	"|     Exy - Pitch slide down                                          |\n"
	"|     Fxy - Pitch slide up                                            |\n"
	"|     Gxx - Glissando                                                 |\n"
	"|     Hxy - Vibrato                                                   |\n"
	"|     Ixx - Tremor                 NOT SUPPORTED, REPURPOSED          |\n"
	"|     Jxy - Arpeggio               NOT SUPPORTED                      |\n"
	"|     Kxy - Volume slide+Vibrato   NOT SUPPORTED, REPURPOSED          |\n"
	"|     Lxy - Volume slide+Glissando NOT SUPPORTED, REPURPOSED          |\n"
	"|     Mxx - Set channel volume     NOT SUPPORTED, REPURPOSED          |\n"
	"|     Nxx - Channel volume slide   NOT SUPPORTED, REPURPOSED          |\n"
	"|     Oxx - Sample offset          NOT SUPPORTED                      |\n"
	"|     Pxy - Panning slide                                             |\n"
	"|     Qxy - Retrigger note         PARTIAL SUPPORT, only takes Y value|\n"
	"|     Rxy - Tremolo                                                   |\n"
	"|     S0x - Set filter             NOT SUPPORTED, REPURPOSED          |\n"
	"|     S1x - Set glissando ctrl     NOT SUPPORTED, REPURPOSED          |\n"
	"|     S2x - Set finetine           NOT SUPPORTED, REPURPOSED          |\n"
	"|     S33 - Random vibrato wave    NOT SUPPORTED, REPURPOSED          |\n"
	"|     S43 - Random tremolo wave    NOT SUPPORTED, REPURPOSED          |\n"
	"|     S5x - Set panbrello wave     NOT SUPPORTED, REPURPOSED          |\n"
	"|     S6x - Pattern delay (ticks)  NOT SUPPORTED, REPURPOSED          |\n"
	"|     S7x - New note action        NOT SUPPORTED                      |\n"
	"|     S8x - Set panning position   NOT SUPPORTED, REPURPOSED          |\n"
	"|     S9x - not supported          NOT SUPPORTED, REPURPOSED          |\n"
	"|     SAy - not supported          NOT SUPPORTED, REPURPOSED          |\n"
	"|     SBx - Pattern loop           NOT SUPPORTED                      |\n"
	"|     SCx - Note cut               NOT SUPPORTED                      |\n"
	"|     SDx - Note delay             NOT SUPPORTED                      |\n"
	"|     SEx - Pattern delay (row)    NOT SUPPORTED, REPURPOSED          |\n"
	"|     SFx - Set parameter MIDI     NOT SUPPORTED, REPURPOSED          |\n"
	"|     T0y - Tempo slide down                                          |\n"
	"|     T1y - Tempo slide up                                            |\n"
	"|     Txy - Tempo                                                     |\n"
	"|     Uxy - Fine vibrato           NOT SUPPORTED, REPURPOSED          |\n"
	"|     Vxy - Global volume          NOT SUPPORTED                      |\n"
	"|     Wxy - Global volume slide                                       |\n"
	"|     Yxy - Panbrello              NOT SUPPORTED, REPURPOSED          |\n"
	"|     Zxx - MIDI Macros            NOT SUPPORTED, REPURPOSED          |\n"
	"|                                                                     |\n"
	"| * '+++' patterns ARE supported!!!                                   |\n"
	"|:::::::::::. ............................................ :::::::::::|\n"
	"|  SET SR  ::                                             ::  SET SR  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Cxx ::  ABNoFX                                 ABMES                |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"|  C00-CFF - Set Sustain Level/Sutain Rate (%%lllrrrrr)                |\n"
	"|                                                                     |\n"
	"|           l - Sustain level                                         |\n"
	"|           r - Sustain rate                                          |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::. ...................... .:::::::::::::::::::::|\n"
	"|  SET ECHO FEEDBACK  ::                       ::  SET ECHO FEEDBACK  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Ixx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| I00-I7F - Set echo feedback                                         |\n"
	"| I80     - Reset echo feedback to the default set in song message    |\n"
	"| I81-IFF - Set echo feedback negative values                         |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::::. ......... .::::::::::::::::::::::::::::|\n"
	"|  SET MAXIMUM ECHO VOLUME  ::           ::  SET MAXIMUM ECHO VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Kxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| K00-K7F - Set minimum echo volume for volume sweeps                 |\n"
	"| K80-KFF - Set echo volume speed                              (-$80) |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::::. ......... .::::::::::::::::::::::::::::|\n"
	"|  SET MIN/MAX NOISE FREQ   ::           ::  SET MINIMUM ECHO VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Lxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| L00-L1F - Set minimum noise frequency for frequency sweeps          |\n"
	"| L20-L3F - Set maximum noise frequency for frequency sweeps   (-$20) |\n"
	"| L80-LFF - Set minimum echo volume for volume sweeps          (-$80) |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::::::. ..... .::::::::::::::::::::::::::::::|\n"
	"|  WAVETABLE/PHASE INVERSION  ::       ::  WAVETABLE/PHASE INVERSION  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Mxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| M00-M3F - Speed at which to change wavetables                       |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::::::. ..... .::::::::::::::::::::::::::::::|\n"
	"|  WAVETABLE/PHASE INVERSION  ::       ::  WAVETABLE/PHASE INVERSION  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Mxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| M40-M47 - Set minimum wavetable                              (-$40) |\n"
	"| M48-M4F - Set maximum wavetable                              (-$48) |\n"
	"|     M51 - Enable phase inversion for one channel                    |\n"
	"|     M52 - Disable phase inversion for one channel                   |\n"
	"|     M53 - Enable phase inversion for all channels                   |\n"
	"|     M54 - Disable phase inversion for all channels                  |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::. ........... .:::::::::::::::::::::::::::|\n"
	"|  ENABLE ADSR AND SET AD  ::             ::  ENABLE ADSR AND SET AD  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Mxx ::  ABNoFX                                 ABMES                |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| M55-M7E - Enable ADSR for current channel* and set Attack and Decay |\n"
	"|           rate (*ABMES M80-MFF only)                                |\n"
	"|     M7F - Use direct gain mode for the current channel [default]    |\n"
	"| M80-MFF - Enable ADSR for current channel and set Attack and Decay  |\n"
	"|          rate (%%Edddaaaa)                                           |\n"
	"|                                                                     |\n"
	"|          E - Enable ADSR                                            |\n"
	"|          d - Decay rate: R=16+d*2                                   |\n"
	"|          a - Attack rate: R=a*2+1                                   |\n"
	"|                                                                     |\n"
	"| * Volume commands will be ignored                                   |\n"
	"|                                                                     |\n"
	"|:::::::::::. ........................................... .:::::::::::|\n"
	"|  FILTER  ::                                             ::  FILTER  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Nxx ::  ABNoFX      ABFM                       ABMES                |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"|     N00 - Disable filter sweep                                      |\n"
	"|     N01 - Sweep from standard to bandpass once                      |\n"
	"|     N02 - Sweep from standard to bandpass pingpong                  |\n"
	"|     N03 - Sweep from bandpass to standard once                      |\n"
	"|     N04 - Sweep from bandpass to standard pingpong                  |\n"
	"|                                                                     |\n"
	"|:::::::::::. ........................................... .:::::::::::|\n"
	"|  FILTER  ::                                             ::  FILTER  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Nxx ::  ABNoFX      ABFM                        ABMES     ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| N05-NFF - Set speed of filter sweep                                 |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::. ........................... .:::::::::::::::::::|\n"
	"|  RETRIGGER NOTE  ::                             ::  RETRIGGER NOTE  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Qxy ::  ABNoFX      ABFM                        ABMES     ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| Q20-Q2F - Code removed to save space. X will always = 2. Y will be  |\n"
	"|           read.                                                     |\n"
	"|                                                                     |\n"
	"|:::::::::. ...............................................  .::::::::|\n"
	"|  ECHO  ::                                                 ::  ECHO  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S01 - Turn on echo for channel                                      |\n"
	"| S02 - Turn off echo for channel                                     |\n"
	"| S03 - Turn on echo for all channels                                 |\n"
	"| S04 - Turn off echo for all channels                                |\n"
	"|                                                                     |\n"
	"| * The other echo settings for the module can be configured during   |\n"
	"|   conversion. See 'SPECIAL SONG MESSAGE COMMANDS'                   |\n"
	"|                                                                     |\n"
	"| * When setting the Echo Delay parameter, you lose '2K x D' of       |\n"
	"|   sample memory from the SPC.                                       |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::. ....................... .:::::::::::::::::::::|\n"
	"|  PITCH MODULATION  ::                         ::  NOISE GENERATION  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S05 - Turn on pitch modulation for one channel                      |\n"
	"| S06 - Turn off pitch modulation for one channel                     |\n"
	"| S07 - Turn on pitch modulation for all channels                     |\n"
	"| S08 - Turn off pitch modulation for all channels                    |\n"
	"| S09 - Turn on noise generation for one channel                      |\n"
	"| S0A - Turn off noise generation for one channel                     |\n"
	"| S0B - Turn on noise generation for all channels                     |\n"
	"| S0C - Turn off noise generation for all channels                    |\n"
	"| S0D - Disable echo writes                                           |\n"
	"| S0E - Enable echo writes                                            |\n"
	"| S0F - Reset echo volume, filter coefficient, and echo feedback      |\n"
	"|       to the values defined in the song message                     |\n"
	"|                                                                     |\n"
	"| If channel filter sweep is enabeled the value current coefficient   |\n"
	"| will be increased or decreased to the value set using Zxx based on  |\n"
	"| the speed set using N05-NFF. If channel filter sweep is disabled    |\n"
	"| this will set the coefficient value immediately.                    |\n"
	"|                                                                     |\n"
	"| * The global volume should always be maximum, or else your pitch    |\n"
	"|   modulation might be weak.                                         |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY  ::                           ::  NOISE FREQUENCY  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S1x-S2x ::  ABNoFX      ABFM                   ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S1x-S2x - Noise frequency, -10h to get your frequency.              |\n"
	"|                                                                     |\n"
	"| * Applies to all channels that have noise enabled.                  |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::. ....................... .:::::::::::::::::::::|\n"
	"|  VIBRATO WAVEFORM  ::                         ::  VIBRATO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S30 - Sine [default]                                                |\n"
	"| S31 - Ramp down                                                     |\n"
	"| S32 - Square                                                        |\n"
	"| S33 - Triangle/Square                                               |\n"
	"| S34 - Triangle                                                      |\n"
	"| S35 - Ramp Up                                                       |\n"
	"| S36 - Square 2                                                      |\n"
	"| S37 - Square 3                                                      |\n"
	"|                                                                     |\n"
	"| * Vibrato and tremolo waveform are set using S3x.                   |\n"
	"|                                                                     |\n"
	"|   When using Hxx, Rxx, and Yxx on the same channel they will use    |\n"
	"|   the same parameters if unspecified. If you use H42 for vibrato    |\n"
	"|   and later use R75 for tremolo, using H00 after the tremolo        |\n"
	"|   command is the same as using H75.                                 |\n"
	"|                                                                     |\n"
	"|::::::::::::::::. ................................. .::::::::::::::::|\n"
	"|  FM WAVEFORM  ::                                   ::  FM WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x ::              ABFM                                            |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S38 - Sine [default]                                                |\n"
	"| S39 - Ramp down                                                     |\n"
	"| S3A - Square                                                        |\n"
	"| S3B - Triangle/Square                                               |\n"
	"| S3C - Triangle                                                      |\n"
	"| S3D - Ramp Up                                                       |\n"
	"| S3E - Square 2                                                      |\n"
	"| S3F - Square 3                                                      |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::. ........................... .:::::::::::::::::::|\n"
	"|  OTHER SETTINGS  ::                             ::  OTHER SETTINGS  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S4x ::              ABFM                                            |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S41 - Disable pitch drop                                            |\n"
	"| S42 - Reset pitch drop                                              |\n"
	"| S43 - Enable pitch drop                                             |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::. ........................... .:::::::::::::::::::|\n"
	"|  OTHER SETTINGS  ::                             ::  OTHER SETTINGS  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S4x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S44 - enable echo volume sweep max->min                             |\n"
	"| S45 - enable echo volume sweep min->max                             |\n"
	"| S46 - Dxy/Gxy is standard volume slide/pitch slide                  |\n"
	"| S47 - Dxy/Gxy is volume/pitch slide with vibrto                     |\n"
	"| S48 - Disable wavetable                                             |\n"
	"| S49 - Enable wavetable                                              |\n"
	"| S4A - Noise sweep decrease                                          |\n"
	"| S4B - Noise sweep increase                                          |\n"
	"| S4C - Noise repeat off                                              |\n"
	"| S4D - Noise repeat on                                               |\n"
	"| S4E - Noise pingpong off                                            |\n"
	"| S4F - Noise pingpong on                                             |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::. ............. .::::::::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY SWEEP  ::               ::  NOISE FREQUENCY SWEEP  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S5x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S5x - Start noise frequency sweep (starting noise +10h)             |\n"
	"|                                                                     |\n"
	"|::::::::::::::::. ................................. .::::::::::::::::|\n"
	"|  SWING TEMPO  ::                                   ::  SWING TEMPO  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S6x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"|     S60 - Disbale swing tempo                                       |\n"
	"| S61-S6F - Enable swing tempo (speed, speed+x)                       |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::. ............. .::::::::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY SPEED  ::               ::  NOISE FREQUENCY SPEED  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S8x ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S8x - Set noise frequency sweep speed                               |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SOUND CONTROL  ::                               ::  SOUND CONTROL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::  ABNoFx      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S90 - Unmute all channels                                           |\n"
	"| S91 - Toggle echo volume (increase|decrease)                        |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SOUND CONTROL  ::                               ::  SOUND CONTROL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::  ABNoFx      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S92 - Mute channel for pitch modulation                             |\n"
	"| S93 - Unmute channel                                                |\n"
	"| S96 - Set ZMode channel filter sweep echo volume maximum -> minimum |\n"
	"| S97 - Set ZMode channel filter sweep echo volume minimum -> maximum |\n"
	"| S98 - Set ZMode channel filter sweep                                |\n"
	"| S99 - Set ZMode sets immediate coefficient value                    |\n"
	"| S9A - Set filter to standard                                        |\n"
	"| S9B - Set filter to bandpass                                        |\n"
	"| S9C - Set filter to highpass                                        |\n"
	"| S9D - Set filter to lowpass                                         |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SOUND CONTROL  ::                               ::  SOUND CONTROL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::              ABFM                                            |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S94 - Enable FM                                                     |\n"
	"| S95 - Disable FM                                                    |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SOUND CONTROL  ::                               ::  SOUND CONTROL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::  ABNoFx      ABFM                       ABMES                |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S9E - Set filter to lowpass2                                        |\n"
	"| S9F - \"                                                             |\n"
	"|                                                                     |\n"
	"| * It is not necessary to mute the channel providing the pitch       |\n"
	"|   factor.                                                           |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  SOUND CONTROL 2  ::                           ::  SOUND CONTROL 2  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| SAx ::  ABNoFx      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| SA0-SA9 - Enable channel filter sweep                               |\n"
	"|     SAA - Enable channel filter sweep and set filter to standard    |\n"
	"|     SAB - Enable channel filter sweep and set filter to bandpass    |\n"
	"|     SAC - Enable channel filter sweep and set filter to highpass    |\n"
	"|     SAD - Enable channel filter sweep and set filter to lowpass     |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  SOUND CONTROL 2  ::                           ::  SOUND CONTROL 2  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| SAx ::  ABNoFx      ABFM                       ABMES                |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"|     SAE - Enable channel filter sweep and set filter to lowpass2    |\n"
	"|     SAF - \"                                                         |\n"
	"|                                                                     |\n"
	"|:::::::::::::::. ................................... .:::::::::::::::|\n"
	"|  ECHO DELAY  ::                                     ::  ECHO DELAY  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| SEx ::  ABNoFx      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| SEx - Change echo delay                                             |\n"
	"|                                                                     |\n"
	"| * It may take up to about 240ms until the hardware realizes changes |\n"
	"|   to the EDL register; ie. when re-allocating the buffer size, the  |\n"
	"|   hardware may keep writing up to 30Kbytes according to the old     |\n"
	"|   size.                                                             |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::. ..................... .::::::::::::::::::::::|\n"
	"|  SET MASTER VOLUME  ::                       ::  SET MASTER VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Uxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| U00-UFF - Set master volume                                         |\n"
	"|     U80 - Reset master volume to value set in module message        |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  SET ECHO VOLUME  ::                           ::  SET ECHO VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Yxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| Y00-YFF - Set echo volume                                           |\n"
	"|     Y80 - Reset echo volume to value set in module message          |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SET PARAMETER  ::                               ::  SET PARAMETER  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Zxx ::  ABNoFX      ABFM                       ABMES      ABStream  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| Z00-Z7F - Set/sweep echo coefficient with positive value            |\n"
	"|     Z80 - Stop filter sweeps, reset filter to standard filter       |\n"
	"| Z81-ZFF - Set/sweep echo coefficient with negative value            |\n"
	"|                                                                     |\n"
	"| S96-S99, SAx commands will alter the behavior here.                 |\n"
	"|                                                                     |\n"
	"| If channel filter sweep is enabeled the value current coefficient   |\n"
	"| will be increased or decreased to the value set using Zxx based on  |\n"
	"| the speed set using N05-NFF. If channel filter sweep is disabled    |\n"
	"| this will set the coefficient value immediately.                    |\n"
	".                                                                     .\n"
};

const char CELES_DOC[] = {
	"| This is not a complete IT implementation, here are some IT features |\n"
	"| that you must not use:                                              |\n"
	"|                                                                     |\n"
	"| * New Note Actions are not supported.                               |\n"
	"| * Complex sample mapping is not supported.                          |\n"
	"|     Only the center entry in each instruments sample map will be    |\n"
	"|     used for the entire map! (and the note values are ignored!!!)   |\n"
	"| * Pitch envelope is not supported.                                  |\n"
	"| * No stereo samples!!                                               |\n"
	"| * Envelope sustain must remain on one node only.                    |\n"
	"| * Envelope nodes must be only 1 to 256 ticks apart.                 |\n"
	"| * Envelope carry is not supported.                                  |\n"
	"| * Random volume/panning variations are not supported.               |\n"
	"| * Pitch/Pan separation is not supported.                            |\n"
	"| * 'Old effects' must be set to 'off'.                               |\n"
	"| * Only linear frequency mode is supported.                          |\n"
	"| * Gxx must not share memory with Exx/Fxx.                           |\n"
	"| * Auto-vibrato is not supported.                                    |\n"
	"| * Filters are not supported.                                        |\n"
	"| * Panning envelopes are not supported                               |\n"
	"| * These volume effects are not supported:                           |\n"
	"|     Ex, Fx, Gx, Hx                                                  |\n"
	"| * These effects are partially/not supported or have been            |\n"
	"|   repurposed:                                                       |\n"
	"|     Axx - Set song speed (hex)                                      |\n"
	"|     Bxx - Jump to Order (hex)                                       |\n"
	"|     Cxx - Pattern Break          NOT SUPPORTED, REPURPOSED          |\n"
	"|                                  *Set pattern length instead*       |\n"
	"|     Dxy - Volume slide                                              |\n"
	"|     Exy - Pitch slide down                                          |\n"
	"|     Fxy - Pitch slide up                                            |\n"
	"|     Gxx - Glissando                                                 |\n"
	"|     Hxy - Vibrato                                                   |\n"
	"|     Ixx - Tremor                 NOT SUPPORTED, REPURPOSED          |\n"
	"|     Jxy - Arpeggio                                                  |\n"
	"|     Kxy - Volume slide+Vibrato                                      |\n"
	"|     Lxy - Volume slide+Glissando                                    |\n"
	"|     Mxx - Set channel volume     NOT SUPPORTED, REPURPOSED          |\n"
	"|     Nxx - Channel volume slide   NOT SUPPORTED, REPURPOSED          |\n"
	"|     Oxx - Sample offset          NOT SUPPORTED, REPURPOSED          |\n"
	"|     Pxy - Panning slide                                             |\n"
	"|     Qxy - Retrigger note                                            |\n"
	"|     Rxy - Tremolo                                                   |\n"
	"|     S0x - Set filter             NOT SUPPORTED, REPURPOSED          |\n"
	"|     S1x - Set glissando ctrl     NOT SUPPORTED, REPURPOSED          |\n"
	"|     S2x - Set finetine           NOT SUPPORTED, REPURPOSED          |\n"
	"|     S33 - Random vibrato wave    NOT SUPPORTED, REPURPOSED          |\n"
	"|     S43 - Random tremolo wave    NOT SUPPORTED, REPURPOSED          |\n"
	"|     S5x - Set panbrello wave     NOT SUPPORTED, REPURPOSED          |\n"
	"|     S6x - Pattern delay (ticks)  NOT SUPPORTED                      |\n"
	"|     S7x - New note action        NOT SUPPORTED                      |\n"
	"|     S8x - Set panning position                                      |\n"
	"|     S9x - not supported          NOT SUPPORTED, REPURPOSED          |\n"
	"|     SAy - not supported          NOT SUPPORTED                      |\n"
	"|     SBx - Pattern loop           NOT SUPPORTED                      |\n"
	"|     SCx - Note cut                                                  |\n"
	"|     SDx - Note delay                                                |\n"
	"|     SEx - Pattern delay (row)    NOT SUPPORTED, REPURPOSED          |\n"
	"|     SFx - not supported          Set parameter MIDI Macro           |\n"
	"|     T0y - Tempo slide down                                          |\n"
	"|     T1y - Tempo slide up                                            |\n"
	"|     Txy - Tempo                                                     |\n"
	"|     Uxy - Fine vibrato           NOT SUPPORTED, REPURPOSED          |\n"
	"|     Vxy - Global volume          NOT SUPPORTED, REPURPOSED          |\n"
	"|     Wxy - Global volume slide    NOT SUPPORTED, REPURPOSED          |\n"
	"|     Yxy - Panbrello              NOT SUPPORTED                      |\n"
	"|     Zxx - MIDI Macros            NOT SUPPORTED, REPURPOSED          |\n"
	"|                                                                     |\n"
	"| * '+++' patterns ARE supported!!!                                   |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::::. ....... .:::::::::::::::::::::::::::::|\n"
	"|  SET SUSTAIN RATE & LEVEL  ::         ::  SET SUSTAIN RATE & LEVEL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Cxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"|  C00-FF - Set Sustain Level/Sutain Rate (%%lllrrrrr)                 |\n"
	"|                                                                     |\n"
	"|           l - Sustain level                                         |\n"
	"|           r - Sustain rate                                          |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  ECHO FEEDBACK  ::                               ::  ECHO FEEDBACK  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Ixx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| I00-7F - Set echo feedback                                          |\n"
	"| I80    - Reset echo feedback to the default set in module message   |\n"
	"| I81-FF - Set echo feedback negative values                          |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::. ........... .:::::::::::::::::::::::::::|\n"
	"|  ENABLE ADSR AND SET AD  ::             ::  ENABLE ADSR AND SET AD  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Mxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| M00-3F - Set wavetable cycle speed                                  |\n"
	"| M40-47 - Set minimum wavetable               (-$40; default is $00) |\n"
	"| M48-4F - Set maximum wavetable               (-$48; default is $07) |\n"
	"| M50-6F - Set echo volume speed                               (-$50) |\n"
	"| M70-7E - Set swing tempo (value added to     (-$70; default is $00) |\n"
	"|                           base speed)                               |\n"
	"|    M7F - Use direct gain mode for the current channel [default]     |\n"
	"| M80-FF - Enable ADSR** for current channel and set the attack and   |\n"
	"|          decay rate (%%Adddaaaa)                                     |\n"
	"|                                                                     |\n"
	"| *  There are no checks when setting the minium and maximum values.  |\n"
	"|    Changing these while the wavetable cycle is active could produce |\n"
	"|    unexpected results. These results may vary based on what the     |\n"
	"|    wavetable cycle speed is set to.                                 |\n"
	"|                                                                     |\n"
	"| ** Volume commands will be ignored                                  |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  NOISE OPTIONS  ::                               ::  NOISE OPTIONS  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Nxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| N00-1F - Set minimum noise frequency for frequency sweeps           |\n"
	"| N20-3F - Set maximum noise frequency for frequency sweeps    (-$20) |\n"
	"| N40-7F - Set noise frequency sweep speed                  (-$40,*4) |\n"
	"|                                                                     |\n"
	"|::::::::::::::::. ................................. .::::::::::::::::|\n"
	"|  ECHO VOLUME  ::                                   ::  ECHO VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Oxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| O00-7F - Set minimum echo volume for volume sweeps                  |\n"
	"| O80    - there is no check for this command it will set the maximum |\n"
	"|          echo volume to 80h which should be avoided                 |\n"
	"| O81-FF - Set maximum echo volume for volume sweeps                  |\n"
	"|                                                                     |\n"
	"|:::::::::. ...............................................  .::::::::|\n"
	"|  ECHO  ::                                                 ::  ECHO  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S01 - Turn on echo for channel                                      |\n"
	"| S02 - Turn off echo for channel                                     |\n"
	"| S03 - Turn on echo for all channels                                 |\n"
	"| S04 - Turn off echo for all channels                                |\n"
	"|                                                                     |\n"
	"| * The other echo settings for the module can be configured during   |\n"
	"|   conversion. See 'SPECIAL SONG MESSAGE COMMANDS'                   |\n"
	"|                                                                     |\n"
	"| * When setting the Echo Delay parameter, you lose '2K x D' of       |\n"
	"|   sample memory from the SPC.                                       |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::. ....................... .:::::::::::::::::::::|\n"
	"|  PITCH MODULATION  ::                         ::  NOISE GENERATION  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S05 - Turn on pitch modulation for one channel                      |\n"
	"| S06 - Turn off pitch modulation for one channel                     |\n"
	"| S07 - Turn on pitch modulation for all channels                     |\n"
	"| S08 - Turn off pitch modulation for all channels                    |\n"
	"| S09 - Turn on noise generation for one channel                      |\n"
	"| S0A - Turn off noise generation for one channel                     |\n"
	"| S0B - Turn on noise generation for all channels                     |\n"
	"| S0C - Turn off noise generation for all channels                    |\n"
	"|                                                                     |\n"
	"|::::::::::. ............................................. .::::::::::|\n"
	"|  RESET  ::                                               ::  RESET  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S0x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S0D - Disable echo write                                            |\n"
	"| S0E - Enable echo write                                             |\n"
	"| S0F - Reset EVOL, EFB, EFIR, and disable filter sweep               |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY  ::                           ::  NOISE FREQUENCY  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S1x-S2x ::                                                          |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S1x-S2x - Noise frequency, -10h to get your frequency. This         |\n"
	"|           automatically disables noise frequency sweep.             |\n"
	"|                                                                     |\n"
	"| * Applies to all channels that have noise enabled.                  |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::. ....................... .:::::::::::::::::::::|\n"
	"|  VIBRATO WAVEFORM  ::                         ::  VIBRATO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S3x  ::                                                             |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S30 - Sine [default]                                                |\n"
	"| S31 - Ramp down                                                     |\n"
	"| S32 - Square                                                        |\n"
	"| S33 - Triangle/Square                                               |\n"
	"| S34 - Triangle                                                      |\n"
	"| S35 - Ramp Up                                                       |\n"
	"| S36 - Square 2                                                      |\n"
	"| S37 - Square 3                                                      |\n"
	"|                                                                     |\n"
	"| * Vibrato, tremolo, and panbrello waveform are set using S3x.       |\n"
	"|   The S4x commands are available for IT compatibility. S5x used for |\n"
	"|   noise.                                                            |\n"
	"|                                                                     |\n"
	"|   When using Hxx, Rxx, and Yxx on the same channel they will use    |\n"
	"|   the same parameters if unspecified. If you use H42 for vibrato    |\n"
	"|   and later use R75 for tremolo, using H00 after the tremolo        |\n"
	"|   command is the same as using H75.                                 |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::. ............. .::::::::::::::::::::::::::|\n"
	"|  FM + VIBRATO WAVEFORM  ::               ::  FM + VIBRATO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S38 - Sine                                                          |\n"
	"| S39 - Ramp down                                                     |\n"
	"| S3A - Square                                                        |\n"
	"| S3B - Triangle/Square                                               |\n"
	"| S3C - Triangle                                                      |\n"
	"| S3D - Ramp Up                                                       |\n"
	"| S3E - Square 2                                                      |\n"
	"| S3F - Square 3                                                      |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::. ............... .:::::::::::::::::::::::::|\n"
	"|  SET TREMOLO WAVEFORM  ::                 ::  SET TREMOLO WAVEFORM  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S4x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S40 - Sine [default]                                                |\n"
	"| S41 - Ramp down                                                     |\n"
	"| S42 - Square                                                        |\n"
	"| S43 - Triangle/Square                                               |\n"
	"|                                                                     |\n"
	"| * See note after vibrato commands.                                  |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  MISC COMMANDS  ::                               ::  MISC COMMANDS  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S4x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S44 - EVOL max -> min                                               |\n"
	"| S45 - EVOL min -> max [default]                                     |\n"
	"| S46 - Enable vibrato during glissando on all channels               |\n"
	"| S47 - Disable vibrato during glissando [default]                    |\n"
	"| S48 - Enable wavetable cycle. This replaces the second sample,      |\n"
	"|       which should have a sample rate of 33488Hz and a length of    |\n"
	"|       64, with wavetable samples. Up to 8 brr samples can be        |\n"
	"|       uploaded using spcChangeWaveTable.                            |\n"
	"| S49 - Disable wavetable cycle [default] It is not possible to       |\n"
	"|       restore the original sample at this time.                     |\n"
	"| S4A - Noise frequency increase                                      |\n"
	"| S4B - Noise frequency decrease [default]                            |\n"
	"| S4C - Noise frequency repeat on                                     |\n"
	"| S4D - Noise frequency repeat off [default]                          |\n"
	"| S4E - Noise frequency ping pong on                                  |\n"
	"| S4F - Noise frequency ping pong off [default]                       |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::::::::. ............. .::::::::::::::::::::::::::|\n"
	"|  NOISE FREQUENCY SWEEP  ::               ::  NOISE FREQUENCY SWEEP  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S5x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S50-S5F - Start noise frequency sweep (-40h = starting frequency)   |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SOUND CONTROL  ::                               ::  SOUND CONTROL  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| S9x ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| S90 - Swap echo volume increase/decrease                            |\n"
	"| S91 - Surround sound                                                |\n"
	"| S92 - Mute channel for pitch modulation                             |\n"
	"| S93 - Unmute channel                                                |\n"
	"| S94 - Enable FM + vibrato. This replaces the first sample which     |\n"
	"|       should have a sample rate of 33488Hz with a length of 64 with |\n"
	"|       FM generated waveforms. These waveforms will only be          |\n"
	"|       generated when S3[8-F] has been set for a channel. Once this  |\n"
	"|       has been set you can generate waveforms by using the vibrato  |\n"
	"|       command (Hxy, x = speed & modulation frequency, y = depth &   |\n"
	"|       carrier frequency.)                                           |\n"
	"| S95 - Disable FM [default]                                          |\n"
	"| S96 - Set ZMODE to filter sweep + enable echo volume sweep min->max |\n"
	"| S97 - Set ZMODE to filter sweep + enable echo volume sweep max->min |\n"
	"| S98 - Set ZMODE to filter sweep                                     |\n"
	"| S99 - Set ZMODE to EFIR [default]                                   |\n"
	"| S9A - Standard filter  [$7f,$00,$00,$00,$00,$00,$00,$00]            |\n"
	"| S9B - Bandpass filter  [$34,$33,$00,$d9,$e5,$01,$fc,$eb]            |\n"
	"| S9C - Highpass filtr   [$58,$bf,$db,$f0,$fe,$07,$0c,$0c]            |\n"
	"| S9D - Lowpass filter   [$0a,$17,$23,$29,$12,$fe,$f3,$f9]            |\n"
	"| S9E -                  [$f8,$08,$11,$1c,$1c,$11,$08,$f8]            |\n"
	"| S9F -                  [$0d,$22,$22,$24,$11,$f0,$03,$ff]            |\n"
	"|                                                                     |\n"
	"|:::::::::::::::. ................................... .:::::::::::::::|\n"
	"|  ECHO DELAY  ::                                     ::  ECHO DELAY  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| SEx - Change echo delay                                             |\n"
	"|                                                                     |\n"
	"| * It may take up to about 240ms until the hardware realizes changes |\n"
	"|   to the EDL register; ie. when re-allocating the buffer size, the  |\n"
	"|   hardware may keep writing up to 30Kbytes according to the old     |\n"
	"|   size.                                                             |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::. ................... .:::::::::::::::::::::::|\n"
	"|  FILTER SWEEP SPEED  ::                     ::  FILTER SWEEP SPEED  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Uxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| U00-FF - Set filter sweep speed                                     |\n"
	"|                                                                     |\n"
	":::::::::::::::::::::::. ..................... .::::::::::::::::::::::|\n"
	"|  SET MASTER VOLUME  ::                       ::  SET MASTER VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Vxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| V00-FF - Set master volume                                          |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::::. ......................... .::::::::::::::::::::|\n"
	"|  SET ECHO VOLUME  ::                           ::  SET ECHO VOLUME  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Yxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| Y00-FF - Set echo volume                                            |\n"
	"|                                                                     |\n"
	"|::::::::::::::::::. ............................. .::::::::::::::::::|\n"
	"|  SET PARAMETER  ::                               ::  SET PARAMETER  |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"| Zxx ::                                                              |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                                                                     |\n"
	"| [S96|S97|S98]                                                       |\n"
	"|       Z80    - Disable channel filter sweep                         |\n"
	"|       Zxx    - Sweep to this value* on the current channel          |\n"
	"|                                                                     |\n"
	"| [S99] Z80    - Reset coefficients to default                        |\n"
	"|       Zxx    - Set coefficient value* on the current channel        |\n"
	"|                                                                     |\n"
	"| * Values >80h are negative numbers.                                 |\n"
	".                                                                     .\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
};

const char TIPS[] = {
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	":    WARNING  WARNING  WARNING  WARNING  WARNING  WARNING  WARNING    :\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	".                                                                     .\n"
	"| * BE EXTREMELY CAREFUL WITH THE ECHO COEFFICIENT VALUES! IT'S       |\n"
	"|   POSSIBLE TO CAUSE PERMENENT DAMAGE TO YOUR EARS OR YOUR SPEAKERS. |\n"
	"|                                                                     |\n"
	"|   DO NOT FUCK AROUND WITH THIS SHIT WHILE WEARING EARBUDS OR        |\n"
	"|   HEADPHONES.                                                       |\n"
	"|                                                                     |\n"
	"|   From the Fullsnes - Nocash SNES Specs:                            |\n"
	"|                                                                     |\n"
	"|     To avoid addition overflows: The sum of POSITIVE values in the  |\n"
	"|     first seven registers (FIR0..FIR6) should not exceed +7Fh, and  |\n"
	"|     the sum of NEGATIVE values should not exceed -7Fh. The sum of   |\n"
	"|     all eight registers (FIR0..FIR7) should be usually around +80h  |\n"
	"|     (for leaving the overall output volume unchanged by the FIR     |\n"
	"|     unit; instead, echo volumes are usually adjusted via EFB/EVOL   |\n"
	"|     registers).                                                     |\n"
	"|                                                                     |\n"
	"| You can use overflow for distortion effects but do not use echo     |\n"
	"| volume sweeps and set the echo volume to 1 or -1. Depending on the  |\n"
	"| feedback settings you can get some different tones/effects but be   |\n"
	"| EXTREMELY careful if you decide to increase the echo volume. Take   |\n"
	"| care to increase by *very* small amounts.                           |\n"
	"|                                                                     |\n"
	"| Standard echo:                                                      |\n"
	"|   $7f,$00,$00,$00,$00,$00,$00,$00  127 0 0 0 0 0 0 0                |\n"
	"|                                                                     |\n"
	"| Bandpass:                                                           |\n"
	"|   $34,$33,$00,$d9,$e5,$01,$fc,$eb  52 51 0 -39 -27 1 -4 -21         |\n"
	"|                                                                     |\n"
	"|  Highpass:                                                          |\n"
	"|   $58,$bf,$db,$f0,$fe,$07,$0c,$0c  -168 -65 -37 -16 -2 7 12 12      |\n"
	"|                                                                     |\n"
	"|  Lowpass:                                                           |\n"
	"|   $0a,$17,$23,$29,$12,$fe,$f3,$f9  10 23 35 41 18 -2 -13 -7         |\n"
	"|                                                                     |\n"
	"|  Lowpass:                                                           |\n"
	"|   $f8,$08,$17,$24,$24,$17,$08,$f8  -8 8 23 36 36 23 8 -8            |\n"
	"|                                                                     |\n"
	"|  Ren & Stimpy:                                                      |\n"
	"|   $f8,$08,$11,$1c,$1c,$11,$08,$f8  -8 8 17 28 28 17 8 -8            |\n"
	"|                                                                     |\n"
	"|  Star Ocean:                                                        |\n"
	"|   $0d,$22,$22,$24,$11,$f0,$03,$ff  13 34 34 36 17 -16 3 -1          |\n"
	"|                                                                     |\n"
	".                                                                     .\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	":  NOTICE    NOTICE    NOTICE    NOTICE    NOTICE    NOTICE    NOTICE :\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	".                                                                     .\n"
	"| * Pitch modulation and noise generation are not completely safe for |\n"
	"|   video games because there aren't any checks for the sound effects |\n"
	"|   on the last two channels (it's especially bad if noise were to be |\n"
	"|   on those two channels... although pitch modulation can allow for  |\n"
	"|   some unusual effects to the music through the SFX if you want     |\n"
	"|   really unusual magic.).                                           |\n"
	"|                                                                     |\n"
	"| * Special commands such as Sxx and Zxx may overlap with unofficial  |\n"
	"|   add-on effects that have been applied since the original Impulse  |\n"
	"|   Tracker, however, SNESMod doesn't support those effects in the    |\n"
	"|   first place. They might affect playback when you're originally    |\n"
	"|   making the song in an unintended fashion, just to note.           |\n"
	".                                                                     .\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	":                        .: SAMPLE QUALITY .:                         :\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	".                                                                     .\n"
	"| 16-bit quality is recommended for the SPC samples, since they have  |\n"
	"| more data for compressing.                                          |\n"
	"|                                                                     |\n"
	"| If the loop width of an SPC sample is not a multiple of 16 then the |\n"
	"| sample will be unrolled OR resampled with linear interpolation for  |\n"
	"| alignment. (it will be resampled if unrolling uses too much memory) |\n"
	"|                                                                     |\n"
	"| Do not downsample your SPC samples too much or else they will sound |\n"
	"| like butt.                                                          |\n"
	"|                                                                     |\n"
	"| Sample offset is supported for the SPC samples, but there is a      |\n"
	"| small chance of creating unwanted NOISE because of the compression. |\n"
	"|                                                                     |\n"
	"| ALSO NOTE BIDI SAMPLES WILL BE UNROLLED AND USE UP LOTS OF SPACE.   |\n"
	"|                                                                     |\n"
	"| *ALSO* If you are making songs for a game remember that channel 8   |\n"
	"| will be OVERWRITTEN with sound effects. Don't put important stuff   |\n"
	"| there. This applies to SNESMod, Pitchmod, and Super SNESMod.        |\n"
	".                                                                     .\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	":                 .: SPECIAL SONG MESSAGE COMMANDS .:                 :\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	".                                                                     .\n"
	"| To control the default echo parameters for a module you must        |\n"
	"| specify some things in the song message.                            |\n"
	"|                                                                     |\n"
	"| To trigger this you must put an '[[SNESMOD]]' tag in your song      |\n"
	"| message followed by special commands!                               |\n"
	"|                                                                     |\n"
	"| See the example IT module, it has song message commands!            |\n"
	"|                                                                     |\n"
	"| Command List:                                                       |\n"
	"|                                                                     |\n"
	"| EDL <delay value>                                                   |\n"
	"|                                                                     |\n"
	"|   This sets the echo delay value. Range is 0-15                     |\n"
	"|   Delay time will be d*16 milliseconds.                             |\n"
	"|   NOTE: each 16ms of delay will use 2K of SPC memory. Your song's   |\n"
	"|   size limit will be more restricted with larger amounts of delay   |\n"
	"|   used.                                                             |\n"
	"|                                                                     |\n"
	"|   Issuing this command will enable echo (master) for the song       |\n"
	"|                                                                     |\n"
	"|   Example:                                                          |\n"
	"|                                                                     |\n"
	"|     'EDL 5'                                                         |\n"
	"|                                                                     |\n"
	"|   The echo delay time will be 5*16 (80) milliseconds. 10 Kilobytes  |\n"
	"|   will be taken away from the memory space, so it will have some-   |\n"
	"|   thing like 48 KB left.                                            |\n"
	"|                                                                     |\n"
	"| EFB <feedback level>                                                |\n"
	"|                                                                     |\n"
	"|   Set echo feedback level. Range is -128..127 (-100%%..100%%)         |\n"
	"|   Default value (if no command present) is 0.                       |\n"
	"|                                                                     |\n"
	"|   Example:                                                          |\n"
	"|                                                                     |\n"
	"|     'EFB 127'                                                       |\n"
	"|                                                                     |\n"
	"|   The echo feedback level will be set to 127, or 100%%.              |\n"
	"|   This value will probably destroy the sound because the feedback   |\n"
	"|   will cover the output.                                            |\n"
	"|                                                                     |\n"
	"| EVOL <volume> [volume right]                                        |\n"
	"|                                                                     |\n"
	"|   Set echo volume level. Range is -128..127.                        |\n"
	"|   One or two values may be given. If only one is specified then it  |\n"
	"|   will be used for both left and right outputs. If two values are   |\n"
	"|   given then one will be used for each output (left, right)         |\n"
	"|                                                                     |\n"
	"|   Examples:                                                         |\n"
	"|                                                                     |\n"
	"|     'EVOL 40'                                                       |\n"
	"|                                                                     |\n"
	"|   Both left and right echo outputs will have a volume of 40,        |\n"
	"|   or 31%%.                                                           |\n"
	"|                                                                     |\n"
	"|     'EVOL 31 -31'                                                   |\n"
	"|                                                                     |\n"
	"|   Left output will be 25%%, right output will be -25%%                |\n"
	"|                                                                     |\n"
	"|   Negative values will indeed inverse the phase of the applied      |\n"
	"|   waveform.                                                         |\n"
	"|                                                                     |\n"
	"| EFIR <c0> [c1] [c2] [c3] [c4] [c5] [c6] [c7]                        |\n"
	"|                                                                     |\n"
	"|   This is ignored by all AB* drivers. Default filter is the same    |\n"
	"|   as noted below. EFIR in AB* drivers can be set in the module.     |\n"
	"|   See the command Z for reference.                                  |\n"
	"|                                                                     |\n"
	"|   Specifies the coefficients for the 8-tap FIR filter applied to    |\n"
	"|   the echo output.                                                  |\n"
	"|                                                                     |\n"
	"|   Default values are 127, 0, 0, 0, 0, 0, 0, 0                       |\n"
	"|                                                                     |\n"
	"|   If less than 8 values are specified then the default values will  |\n"
	"|   be used for the rest of the entries.                              |\n"
	"|                                                                     |\n"
	"|   Example:                                                          |\n"
	"|                                                                     |\n"
	"|     'EFIR 64 -32 16'                                                |\n"
	"|                                                                     |\n"
	"|   Set the first three coefficients to these values and the rest     |\n"
	"|   will be zero.                                                     |\n"
	"|                                                                     |\n"
	"| EON <channels>                                                      |\n"
	"|                                                                     |\n"
	"|   Enable echo for the channels listed.                              |\n"
	"|                                                                     |\n"
	"|   Example:                                                          |\n"
	"|                                                                     |\n"
	"|     'EON 1 3 4 5'                                                   |\n"
	"|                                                                     |\n"
	"|   Enable echo for channels 1 (first), 3, 4, and 5.                  |\n"
	"|                                                                     |\n"
	"| Here is an example song message with commands in it:                |\n"
	"|---------------------------------------------------------------------|\n"
	"| Here is my song, blah blah blah blah                                |\n"
	"| By Ms.X                                                             |\n"
	"|                                                                     |\n"
	"| [[SNESMOD]]                                                         |\n"
	"| edl 6                                                               |\n"
	"| efb 127                                                             |\n"
	"| evol 31 -31                                                         |\n"
	"| efir 127 0 0 0 0 0 0 0                                              |\n"
	"| eon 1 2 3                                                           |\n"
	".                                                                     .\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	":                       .: SPACE SAVING TIPS .:                       :\n"
	":::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::\n"
	".                                                                     .\n"
	"| * Remove commands that are not used. If you have commands and       |\n"
	"|   notes after the break pattern command you are wasting bytes.      |\n"
	"|                                                                     |\n"
	"| * Remove commands that are duplicated. It's not necessary to set    |\n"
	"|   the tempo on every pattern if you are not changing the value.     |\n"
	"|                                                                     |\n"
	"| * Volume envelopes can save a lot of space over using the volume    |\n"
	"|   commands.                                                         |\n"
	"|                                                                     |\n"
	"| * Don't bother with the pattern break command. Just set the         |\n"
	"|   pattern length.                                                   |\n"
	"|                                                                     |\n"
	"| * If there are two patterns where lines 00 - 1f are the same and    |\n"
	"|   lines 20-3f are different, cut the patterns in half.              |\n"
	"|                                                                     |\n"
	"|   For example, if you have two patterns 00 and 01, copy 00 to 02.   |\n"
	"|   Set the pattern length to 32. On patterns 00 and 01 move lines    |\n"
	"|   20-3f up to 00-1f and again set the pattern length to 32. Now     |\n"
	"|   the order is 02,00,02,01 instead of 00,01.                        |\n"
	"|                                                                     |\n"
	"| * Is the whole sample being used? If not, consider cutting it down. |\n"
	"|                                                                     |\n"
	"|:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::|\n"
	"|                            (PLEASE ENJOY)                           |\n"
	"`---------------------------------------------------------------------'\n"
	"  -_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-_-\n"

};

#endif // HELP_H
